#include "fvCFD.H" //Used only for code autocompletion

volScalarField psi0("psi0",(double(2.0)*alpha1-double(1.0))*gamma);

// solve Level-Set function as the re-initialization equation
Info<< "solve the reinitialization equation"<< nl << endl;

psi == psi0;

int Ncorr = epsilon.value()/deltaTau.value();
for (int corr=0; corr<Ncorr; corr++)
{
   psi = psi + psi0/mag(psi0)*(double(1)-mag(fvc::grad(psi)*dimChange))*deltaTau;
   psi.correctBoundaryConditions();
}

// update Dirac function
forAll(mesh.cells(),celli)
{
   if(mag(psi[celli]) > epsilon.value())
      delta[celli] = double(0);
   else
      delta[celli] = double(1.0)/(double(2.0)*epsilon.value())*(double(1.0)+Foam::cos(M_PI*psi[celli]/epsilon.value()));
};

// update Heaviside function
forAll(mesh.cells(),celli)
{
   if(psi[celli] < -epsilon.value())
      H[celli] = double(0);
   else if(epsilon.value() < psi[celli])
      H[celli] = double(1);
   else
      H[celli] = double(1.0)/double(2.0)*(double(1.0)+psi[celli]/epsilon.value()+Foam::sin(M_PI*psi[celli]/epsilon.value())/M_PI);
};


// calculate normal vector
//volVectorField gradPsi(fvc::grad(alpha1)); //VoF curvature
volVectorField gradPsi(fvc::grad(psi)); //Level Set function curvature
//volVectorField gradPsi(fvc::grad(H)); //Other authors prefer to use the smoothed Heaviside function
surfaceVectorField gradPsif(fvc::interpolate(gradPsi));
surfaceVectorField nVecfv(gradPsif/(mag(gradPsif)+scalar(1.0e-6)/dimChange));
surfaceScalarField nVecf(nVecfv & mesh.Sf());

// calculate new curvature based on psi (LS function)
C == -fvc::div(nVecf);
